"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const messenger_1 = require("./messenger");
class Connection {
    constructor(name, endpoints = []) {
        this.name = name;
        this.endpoints = endpoints.map(endpoint => {
            if (endpoint.type === 'discord' && !endpoint.channel) {
                endpoint.channel = discord.client.channels.get(endpoint.id);
            }
            return endpoint;
        });
    }
    static isThread(endpoint) { return endpoint.type === 'messenger'; }
    static isChannel(endpoint) { return endpoint.type === 'discord'; }
    addThread({ id, name }) {
        this.endpoints.push({
            type: 'messenger',
            id: id,
            name
        });
        return this;
    }
    addChannel({ id, name }) {
        this.endpoints.push({
            type: 'discord',
            id,
            name,
            channel: discord.client.channels.get(id)
        });
        return this;
    }
    async addEndpoint({ id, type, readonly }) {
        if (type === 'discord') {
            const channel = discord.client.channels.find(channel => channel.id === id || channel.name === id);
            if (!channel)
                throw new Error(`Discord channel \`${id}\` not found!`);
            await this
                .addChannel({
                id: channel.id,
                name: channel.name
            })
                .save();
        }
        else {
            let thread;
            try {
                thread = await messenger_1.getThread(id);
            }
            catch (err) {
                throw new Error(`Messenger thread \`${id}\` not found!`);
            }
            await this
                .addThread({
                id,
                name: thread.name
            })
                .save();
        }
        if (readonly)
            this.markEndpointAsReadonly(id, true);
    }
    has(id) {
        return this.endpoints.some(endpoint => endpoint.id === id);
    }
    getWritableEndpoints() { return this.endpoints.filter(endpoint => !endpoint.readonly); }
    getThreads() { return this.endpoints.filter(Connection.isThread); }
    getWritableThreads() { return this.getWritableEndpoints().filter(Connection.isThread); }
    getOtherWritableThreads(id) { return this.getWritableThreads().filter(thread => thread.id !== id.toString()); }
    getChannels() { return this.endpoints.filter(Connection.isChannel).filter(channel => channel.channel); }
    getWritableChannels() { return this.getWritableEndpoints().filter(Connection.isChannel).filter(channel => channel.channel); }
    getOtherWritableChannels(id) { return this.getWritableChannels().filter(channel => channel.id !== id); }
    async checkChannelRenames(name) {
        if (name &&
            config.discord.renameChannels &&
            this.getThreads().length === 1 &&
            this.getChannels().length === 1 &&
            !this.getChannels()[0].readonly &&
            this.getChannels()[0].name !== name) {
            const channel = this.getChannels()[0];
            await channel.channel.edit({ name });
        }
        return this;
    }
    hasEndpoint(id) {
        return this.endpoints.some(endpoint => endpoint.id === id || endpoint.name === id);
    }
    markEndpointAsReadonly(id, readonly) {
        let endpoint = this.endpoints.find(endpoint => endpoint.id === id || endpoint.name === id);
        endpoint.readonly = readonly;
        return this;
    }
    removeEndpoint(id) {
        let index = this.endpoints.findIndex(endpoint => endpoint.id === id || endpoint.name === id);
        if (index)
            this.endpoints.splice(index, 1);
        return this.save();
    }
    getPrintable() {
        const getLink = ({ type, id }) => type === 'messenger'
            ? `[\`${id}\`](https://facebook.com/messages/t/${id})`
            : `<#${id}>`;
        const e = (endpoint) => `\`${endpoint.type}\`: ${getLink(endpoint)}${endpoint.readonly ? ' (readonly)' : ''}`;
        return this.endpoints.map(e).join('\n');
    }
    rename(newName) {
        connections.list.delete(this.name);
        this.name = newName;
        return this.save();
    }
    delete() {
        connections.list.delete(this.name);
        return connections.save();
    }
    async save() {
        connections.list.set(this.name, this);
        await connections.save();
        return this;
    }
    toYAMLObject() {
        return {
            [this.name]: this.cleanEndpoints
        };
    }
    toObject() {
        return {
            name: this.name,
            endpoints: this.cleanEndpoints
        };
    }
    get cleanEndpoints() {
        return this.endpoints
            .map(({ type, id, name, readonly }) => ({ type, id, name, readonly }))
            .map(endpoint => {
            if (!endpoint.readonly)
                delete endpoint.readonly;
            return endpoint;
        });
    }
}
exports.default = Connection;
