"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const timezonedDate_1 = __importDefault(require("./timezonedDate"));
const util_1 = __importDefault(require("util"));
const chalk_1 = __importDefault(require("chalk"));
const is_docker_1 = __importDefault(require("is-docker"));
const isDocker = is_docker_1.default();
const levelBadges = {
    fatal: chalk_1.default.bgWhite.red('  FATAL  '),
    error: chalk_1.default.bgRed.white('  ERROR  '),
    warn: chalk_1.default.bgYellow.black('  WARN   '),
    info: chalk_1.default.bgBlue.black('  INFO   '),
    start: chalk_1.default.bgBlue.black('  START  '),
    success: chalk_1.default.bgGreen.black(' SUCCESS '),
    debug: chalk_1.default.bgCyan.black('  DEBUG  '),
    trace: chalk_1.default.bgWhite.black('  TRACE  ')
};
const levelArrows = {
    fatal: chalk_1.default.white('›'),
    error: chalk_1.default.red('›'),
    warn: chalk_1.default.yellow('›'),
    info: chalk_1.default.blue('›'),
    start: chalk_1.default.green('›'),
    success: chalk_1.default.green('›'),
    debug: chalk_1.default.cyan('›'),
    trace: chalk_1.default.white('›')
};
const levels = {
    silent: -1,
    fatal: 0,
    error: 1,
    warn: 2,
    info: 3,
    start: 3,
    success: 3,
    verbose: 4,
    debug: 4,
    silly: 5,
    trace: 5
};
class Logger {
    constructor(level, scope) {
        this.level = typeof level === 'number' ? level : levels[level || 'info'];
        this.scopedLoggers = [];
        if (scope) {
            this.scope = scope;
        }
    }
    withScope(scope) {
        const logger = new Logger(this.level, this.scope ? `${this.scope}:${scope}` : scope);
        this.scopedLoggers.push(logger);
        return logger;
    }
    setLevel(level) {
        this.level = levels[level];
        this.scopedLoggers.forEach(logger => logger.setLevel(level));
    }
    getScopeForLevel(level) {
        return (this.scope || '')
            .split(':')
            .map(frag => chalk_1.default.reset(frag))
            .join(` ${levelArrows[level]} `);
    }
    inspect(object, depth = 2, prefixLength = 20) {
        const options = {
            depth,
            colors: true,
            breakLength: (process.stdout.columns - (prefixLength + this.getScopeForLevel('info').length))
        };
        if (isDocker)
            delete options.breakLength;
        return util_1.default.inspect(object, options);
    }
    log(logLevel, msg, obj, depth) {
        if (this.level < levels[logLevel])
            return;
        if (process.env.JSON_LOGS) {
            console.log(JSON.stringify({
                timestamp: new Date().toJSON(),
                name: this.scope,
                level: levels[logLevel],
                ...(typeof msg !== 'object'
                    ? {
                        message: msg,
                        data: obj
                    }
                    : {
                        message: '',
                        data: msg
                    })
            }));
            return;
        }
        let prefix = '';
        if (typeof msg === 'object') {
            msg = chalk_1.default.reset() + this.inspect(msg, obj);
        }
        else if (obj) {
            prefix = chalk_1.default.reset(msg);
            msg = obj
                ? this.inspect(obj, depth, 20 + prefix.length)
                : '';
        }
        else {
            msg = chalk_1.default.reset(msg);
        }
        msg = msg.split('\n').map(line => [
            levelBadges[logLevel],
            chalk_1.default.gray(timezonedDate_1.default().split('T')[1]),
            this.getScopeForLevel(logLevel),
            levelArrows[logLevel],
            prefix,
            line
        ].join(' ')).join('\n');
        console.log(msg);
    }
    fatal(message, object, depth) { this.log('fatal', message, object, depth); }
    error(message, object, depth) { this.log('error', message, object, depth); }
    info(message, object, depth) { this.log('info', message, object, depth); }
    start(message, object, depth) { this.log('start', message, object, depth); }
    success(message, object, depth) { this.log('success', message, object, depth); }
    warn(message, object, depth) { this.log('warn', message, object, depth); }
    debug(message, object, depth) { this.log('debug', message, object, depth); }
    trace(message, object, depth) { this.log('trace', message, object, depth); }
}
exports.default = Logger;
